@@ -1,3 +1,5 @@
+import re
+
 import boto3
 from boto3.dynamodb.conditions import Key, Attr
 from boto3.exceptions import ResourceNotExistsError
@@ -27,6 +29,7 @@ class Response:
     def __init__(self, response_dict: dict):
         response_safedict = SafeDict(response_dict)
         self.items = response_safedict.get("Items").to_list(default=None)
+        self.attributes = response_safedict.get("Attributes").to_dict(default=None)
         self.count = response_safedict.get("Count").to_int(default=None)
         self.scanned_count = response_safedict.get("ScannedCount").to_int(default=None)
         self.last_evaluated_key = response_safedict.get("LastEvaluatedKey").to_dict(default=None)
@@ -60,7 +63,7 @@ class GlobalSecondaryIndex(Index):
 
         super().__init__(
             hash_key_name=hash_key_name, hash_key_variable_python_type=hash_key_variable_python_type,
-            sort_key_name=sort_key_name, sort_key_variable_python_type=sort_key_variable_python_type,
+            sort_index_name=sort_key_name, sort_key_variable_python_type=sort_key_variable_python_type,
             projection_type=projection_type, non_key_attributes=non_key_attributes
         )
 
@@ -152,6 +155,7 @@ class CreateTableQueryKwargs:
 
         self._add_global_secondary_index(key_name=global_secondary_index.hash_key_name,
                                          key_python_variable_type=global_secondary_index.hash_key_variable_python_type)
+
         if global_secondary_index.sort_key_name is not None and global_secondary_index.sort_key_variable_python_type is not None:
             self._add_global_secondary_index(key_name=global_secondary_index.sort_key_name,
                                              key_python_variable_type=global_secondary_index.sort_key_variable_python_type)
@@ -205,7 +209,6 @@ class DynamoDbCoreAdapter:
                              f"The dynamodb client has been initialized without specifying the region.")
 
         self._create_table_if_not_exists()
-        print(f"Initialization of {self} completed successfully !")
 
     def _create_table_if_not_exists(self) -> None:
         """
@@ -229,7 +232,7 @@ class DynamoDbCoreAdapter:
                 self.dynamodb.create_table(**create_table_query_kwargs.data)
             except Exception as e:
                 if e.__class__.__name__ != "ResourceInUseException":
-                    raise Exception(f"Create table if not exists request failed: Exception of type {type(e).__name__} occurred {str(e)}")
+                    raise Exception(f"Request to create the table if not exist failed: Exception of type {type(e).__name__} occurred {str(e)}")
 
     def put_record(self, item_dict: dict) -> bool:
         try:
@@ -253,19 +256,18 @@ class DynamoDbCoreAdapter:
             print(e)
         return False
 
-    def get_item_by_primary_key(self, key_name: str, key_value: any, fields_to_get: Optional[List[str]]) -> Optional[GetItemResponse]:
-        if fields_to_get is not None:
-            kwargs = self._fields_to_get_to_expressions(fields_to_get=fields_to_get)
+    def get_item_by_primary_key(self, index_name: str, key_value: any, fields_paths_elements: Optional[List[List[DatabasePathElement]]]) -> Optional[GetItemResponse]:
+        if fields_paths_elements is not None:
+            kwargs = self._fields_paths_elements_to_expressions(fields_paths_elements=fields_paths_elements)
         else:
             kwargs = dict()
-        kwargs["Key"] = {key_name: key_value}
+        kwargs["Key"] = {index_name: key_value}
         kwargs["ConsistentRead"] = True
 
         try:
             table = self.dynamodb.Table(self.table_name)
             response = table.get_item(**kwargs)
             if "Item" in response:
-                e = Utils.dynamodb_to_python_higher_level(dynamodb_object=response["Item"])
                 return GetItemResponse(item=Utils.dynamodb_to_python_higher_level(dynamodb_object=response["Item"]), success=True)
             else:
                 return GetItemResponse(item=None, success=False)
@@ -276,6 +278,9 @@ class DynamoDbCoreAdapter:
             print(f"Failed to retrieve attributes from DynamoDb table. Exception of type {type(e).__name__} occurred: {str(e)}")
             return None
 
+    def check_if_item_exist_by_primary_key(self, index_name: str, key_value: str, fields_paths_elements: Optional[List[str]]) -> Optional[bool]:
+        raise Exception("Not implemented")  # todo: implement
+
     def _execute_update_query(self, query_kwargs_dict: dict) -> Optional[Response]:
         try:
             table = self.dynamodb.Table(self.table_name)
@@ -291,11 +296,10 @@ class DynamoDbCoreAdapter:
             print(f"Failed to update attributes in DynamoDb table. Exception of type {type(e).__name__} occurred: {str(e)}")
             return None
 
-    def add_data_elements_to_list(self, key_name: str, key_value: Any, object_path: str,
-                                  element_values: List[dict]) -> Optional[Response]:
+    def add_data_elements_to_list(self, index_name: str, key_value: Any, object_path: str, element_values: List[dict]) -> Optional[Response]:
         kwargs = {
             "TableName": self.table_name,
-            "Key": {key_name: key_value},
+            "Key": {index_name: key_value},
             "ReturnValues": "UPDATED_NEW",
             "UpdateExpression": f"SET {object_path} = list_append(if_not_exists({object_path}, :emptyList), :newItems)",
             # The if_not_exists inside the list_append, will create an empty
@@ -307,11 +311,10 @@ class DynamoDbCoreAdapter:
         }
         return self._execute_update_query(query_kwargs_dict=kwargs)
 
-    def remove_data_elements_from_list(self, key_name: str, key_value: Any, list_object_path: str,
-                                       indexes_to_remove: list) -> Optional[Response]:
+    def remove_data_elements_from_list(self, index_name: str, key_value: Any, list_object_path: str, indexes_to_remove: list) -> Optional[Response]:
         kwargs = {
             "TableName": self.table_name,
-            "Key": {key_name: key_value},
+            "Key": {index_name: key_value},
             "ReturnValues": "UPDATED_NEW"
         }
         update_expression = "REMOVE "
@@ -323,8 +326,12 @@ class DynamoDbCoreAdapter:
 
         return self._execute_update_query(query_kwargs_dict=kwargs)
 
-    def remove_data_elements_from_map(self, key_name: str, key_value: Any,
-                                      targets_path_elements: List[List[DatabasePathElement]]) -> Optional[Response]:
+    def remove_data_elements_from_map(
+            self, index_name: str, key_value: Any,
+            targets_path_elements: List[List[DatabasePathElement]],
+            retrieve_removed_elements: bool = False
+    ) -> Optional[Response]:
+
         expression_attribute_names_dict = dict()
         update_expression = "REMOVE "
 
@@ -341,22 +348,35 @@ class DynamoDbCoreAdapter:
                 update_expression += ", "
 
         update_query_kwargs = {
-            "TableName": self.table_name,
-            "Key": {key_name: key_value},
-            "ReturnValues": "UPDATED_NEW",
-            "UpdateExpression": update_expression,
-            "ExpressionAttributeNames": expression_attribute_names_dict,
+            'TableName': self.table_name,
+            'Key': {index_name: key_value},
+            'ReturnValues': "UPDATED_OLD" if retrieve_removed_elements is True else "NONE",
+            'UpdateExpression': update_expression,
+            'ExpressionAttributeNames': expression_attribute_names_dict,
         }
         response = self._execute_update_query(query_kwargs_dict=update_query_kwargs)
+        if response.attributes is not None:
+            response.attributes = Utils.dynamodb_to_python_higher_level(response.attributes)
         return response
 
-    def initialize_all_elements_in_map_target(self, key_name: str, key_value: Any, target_path_elements: List[DatabasePathElement]) -> bool:
+    def initialize_all_elements_in_map_target(
+            self, index_name: str, key_value: Any, field_path_elements: List[DatabasePathElement],
+            last_item_custom_overriding_init_value: Optional[Any] = None
+    ) -> Optional[Response]:
+
         current_path_target = ""
         expression_attribute_names_dict: Dict[str, str] = dict()
-        for i, path_element in enumerate(target_path_elements):
+        last_response: Optional[Response] = None
+
+        for i, path_element in enumerate(field_path_elements):
             if i > 0:
                 current_path_target += "."
 
+            if i + 1 >= len(field_path_elements) and last_item_custom_overriding_init_value is not None:
+                item_default_value = last_item_custom_overriding_init_value
+            else:
+                item_default_value = path_element.get_default_value()
+
             current_path_key = f"#pathKey{i}"
             current_path_target += current_path_key
             expression_attribute_names_dict[current_path_key] = path_element.element_key
@@ -364,36 +384,36 @@ class DynamoDbCoreAdapter:
             current_update_expression = f"SET {current_path_target} = if_not_exists({current_path_target}, :item)"
             current_set_potentially_missing_object_query_kwargs = {
                 "TableName": self.table_name,
-                "Key": {key_name: key_value},
+                "Key": {index_name: key_value},
                 "ReturnValues": "UPDATED_NEW",
                 "UpdateExpression": current_update_expression,
                 "ExpressionAttributeNames": expression_attribute_names_dict,
                 "ExpressionAttributeValues": {
-                    ":item": path_element.get_default_value()
+                    ":item": item_default_value
                 }
             }
-            response = self._execute_update_query(query_kwargs_dict=current_set_potentially_missing_object_query_kwargs)
-            if response is None:
-                return False
-        return True
+            last_response = self._execute_update_query(query_kwargs_dict=current_set_potentially_missing_object_query_kwargs)
+        return last_response
 
-    def set_update_data_element_to_map(self, key_name: str, key_value: Any, value: Any,
-                                       target_path_elements: List[DatabasePathElement]) -> Optional[Response]:
+    def set_update_data_element_to_map(self, index_name: str, key_value: Any, value: Any,
+                                       field_path_elements: List[DatabasePathElement]) -> Optional[Response]:
         expression_attribute_names_dict = dict()
         update_expression = "SET "
 
-        for i, path_element in enumerate(target_path_elements):
+        last_item_custom_overriding_init_value = None
+        for i, path_element in enumerate(field_path_elements):
             current_path_key = f"#pathKey{i}"
             update_expression += current_path_key
             expression_attribute_names_dict[current_path_key] = path_element.element_key
-            if i + 1 < len(target_path_elements):
+            if i + 1 < len(field_path_elements):
                 update_expression += "."
             else:
                 update_expression += " = :item"
+                last_item_custom_overriding_init_value = value
 
         update_query_kwargs = {
             "TableName": self.table_name,
-            "Key": {key_name: key_value},
+            "Key": {index_name: key_value},
             "ReturnValues": "UPDATED_NEW",
             "UpdateExpression": update_expression,
             "ExpressionAttributeNames": expression_attribute_names_dict,
@@ -403,34 +423,38 @@ class DynamoDbCoreAdapter:
         }
         response = self._execute_update_query(query_kwargs_dict=update_query_kwargs)
         if response is None:
-            # If the response is None, it means that one of the path of the
-            # target path has not been found and need to be initialized.
-            success: bool = self.initialize_all_elements_in_map_target(
-                key_name=key_name, key_value=key_value, target_path_elements=target_path_elements
+            # If the response is None, it means that one of the path of the target path has not been found and need to be initialized.
+            print(message_with_vars(
+                message="Set/update request failed. Trying to initialize and populate the required objects.",
+                vars_dict={'index_name': index_name, 'key_value': key_value, 'field_path_elements': field_path_elements}
+            ))
+            return self.initialize_all_elements_in_map_target(
+                index_name=index_name, key_value=key_value, field_path_elements=field_path_elements,
+                last_item_custom_overriding_init_value=last_item_custom_overriding_init_value
             )
-            if success is True:
-                response = self._execute_update_query(query_kwargs_dict=update_query_kwargs)
         return response
 
-    def _execute_update_query_with_initialization_if_missing(self, key_name: str, key_value: Any, update_query_kwargs: dict,
+    def _execute_update_query_with_initialization_if_missing(self, index_name: str, key_value: Any, update_query_kwargs: dict,
                                                              setters: List[DynamoDBMapObjectSetter]) -> Optional[Response]:
         response = self._execute_update_query(query_kwargs_dict=update_query_kwargs)
         if response is None:
             # If the response is None, it means that one of the path of the
             # target path has not been found and need to be initialized.
+            last_response: Optional[Response] = None
             for i_setter, current_setter in enumerate(setters):
-                success: bool = self.initialize_all_elements_in_map_target(
-                    key_name=key_name, key_value=key_value, target_path_elements=current_setter.target_path_elements
+                last_response = self.initialize_all_elements_in_map_target(
+                    index_name=index_name, key_value=key_value,
+                    field_path_elements=current_setter.field_path_elements
                 )
-                print(message_with_vars(
-                    message="Initialized a field after a set/update multiple data elements in map request had failed.",
-                    vars_dict={"fieldTargetPathElements": current_setter.target_path_elements}
-                ))
-            response = self._execute_update_query(query_kwargs_dict=update_query_kwargs)
+                if last_response is None:
+                    print(message_with_vars(
+                        message="Initialized a field after a set/update multiple data elements in map request had failed.",
+                        vars_dict={"fieldTargetPathElements": current_setter.field_path_elements}
+                    ))
+            return last_response
         return response
 
-    def set_update_multiple_data_elements_to_map(self, key_name: str, key_value: Any,
-                                                 setters: List[DynamoDBMapObjectSetter]) -> Optional[Response]:
+    def set_update_multiple_data_elements_to_map(self, index_name: str, key_value: Any, setters: List[DynamoDBMapObjectSetter]) -> Optional[Response]:
         if not len(setters) > 0:
             # If we tried to run the query with no object setter,
             # she will crash when executed. So we return None.
@@ -438,7 +462,7 @@ class DynamoDbCoreAdapter:
 
         update_query_kwargs = {
             "TableName": self.table_name,
-            "Key": {key_name: key_value},
+            "Key": {index_name: key_value},
             "ReturnValues": "UPDATED_NEW"
         }
         update_expression = "SET "
@@ -451,12 +475,11 @@ class DynamoDbCoreAdapter:
             current_setter_update_expression = ""
             current_setter_attribute_names = dict()
             current_setter_attribute_values = dict()
-
-            for i_path, current_path_element in enumerate(current_setter.target_path_elements):
+            for i_path, current_path_element in enumerate(current_setter.field_path_elements):
                 current_path_key = f"#setter{i_setter}_pathKey{i_path}"
                 current_setter_update_expression += current_path_key
                 current_setter_attribute_names[current_path_key] = current_path_element.element_key
-                if i_path + 1 < len(current_setter.target_path_elements):
+                if i_path + 1 < len(current_setter.field_path_elements):
                     current_setter_update_expression += "."
                 else:
                     current_setter_update_expression += f" = :item{i_setter}"
@@ -483,7 +506,7 @@ class DynamoDbCoreAdapter:
                     message="Update operation expression size has reached over 4kb. "
                             "The operation will be divided in a secondary operation (which could also be divided)",
                     vars_dict={
-                        'key_name': key_name, 'key_value': key_value,
+                        'index_name': index_name, 'key_value': key_value,
                         'setters': setters, 'update_expression': update_expression,
                         'current_setter_update_expression': current_setter_update_expression,
                         'current_setter_attribute_names': current_setter_attribute_names,
@@ -498,7 +521,7 @@ class DynamoDbCoreAdapter:
             update_query_kwargs["ExpressionAttributeNames"] = expression_attribute_names_dict
 
         response = self._execute_update_query_with_initialization_if_missing(
-            key_name=key_name, key_value=key_value,
+            index_name=index_name, key_value=key_value,
             update_query_kwargs=update_query_kwargs, setters=consumed_setters
         )
         if len(consumed_setters) == len(setters):
@@ -506,21 +529,20 @@ class DynamoDbCoreAdapter:
         else:
             for setter in consumed_setters:
                 setters.remove(setter)
-            return self.set_update_multiple_data_elements_to_map(key_name=key_name, key_value=key_value, setters=setters)
+            return self.set_update_multiple_data_elements_to_map(index_name=index_name, key_value=key_value, setters=setters)
 
-
-    def query_by_key(self, key_name: str, key_value: Any, index_name: Optional[str] = None,
-                     fields_to_get: Optional[list] = None, filter_expression: Optional[Any] = None,
-                     query_limit: Optional[int] = None, **additional_kwargs) -> Response:
-
-        if fields_to_get is not None:
-            kwargs = self._fields_to_get_to_expressions(fields_to_get=fields_to_get)
+    def query_by_key(
+            self, index_name: str, key_value: Any, fields_paths_elements: Optional[List[List[DatabasePathElement]]] = None,
+            filter_expression: Optional[Any] = None, query_limit: Optional[int] = None, **additional_kwargs
+    ) -> Response:
+        if fields_paths_elements is not None:
+            kwargs = self._fields_paths_elements_to_expressions(fields_paths_elements=fields_paths_elements)
         else:
             kwargs = dict()
         if additional_kwargs is not None:
             kwargs = {**kwargs, **additional_kwargs}
 
-        kwargs["KeyConditionExpression"] = Key(key_name).eq(key_value)
+        kwargs["KeyConditionExpression"] = Key(index_name).eq(key_value)
         if index_name is not None:
             kwargs["IndexName"] = index_name
 
@@ -541,14 +563,15 @@ class DynamoDbCoreAdapter:
             raise Exception(f"Failed to retrieve attributes from DynamoDb table."
                             f"Exception of type {type(e).__name__} occurred: {str(e)}")
 
-    def query_single_item_by_key(self, key_name: str, key_value: Any, index_name: Optional[str] = None,
-                                 fields_to_get: Optional[list] = None, filter_expression: Optional[Any] = None) -> Optional[dict]:
+    def query_single_item_by_key(self, index_name: str, key_value: Any, fields_paths_elements: Optional[List[List[DatabasePathElement]]] = None,
+                                 filter_expression: Optional[Any] = None) -> Optional[dict]:
         # Yes, a query request is heavier than a get request that we could do with the _get_item_by_primary_key function.
         # Yet, in a get request, we cannot specify an index_name to query on. So, the _query_single_item_by_key should be
         # used when we want to get an item based on another index that the primary one. Otherwise, use _get_item_by_primary_key
         response = self.query_by_key(
-            key_name=key_name, key_value=key_value, index_name=index_name,
-            fields_to_get=fields_to_get, filter_expression=filter_expression, query_limit=1
+            index_name=index_name, key_value=key_value,
+            fields_paths_elements=fields_paths_elements,
+            filter_expression=filter_expression, query_limit=1
         )
         if response.count == 1:
             return response.items[0]
@@ -559,78 +582,102 @@ class DynamoDbCoreAdapter:
             print("More than one item has been found. Returning first item.")
             return response.items[0]
 
-    def _get_or_query_single_item(self, key_name: str, key_value: str, fields_to_get: List[str]) -> Optional[dict]:
-        if self.primary_index.hash_key_name == key_name:
+    def _get_or_query_single_item(self, index_name: str, key_value: str, fields_paths_elements: List[List[DatabasePathElement]]) -> Optional[dict]:
+        if self.primary_index.hash_key_name == index_name:
             response_item: Optional[dict] = self.get_item_by_primary_key(
-                key_name=key_name, key_value=key_value, fields_to_get=fields_to_get
+                index_name=index_name, key_value=key_value, fields_paths_elements=fields_paths_elements
             ).item
             return response_item
         else:
-            if key_name not in self._global_secondary_indexes_hash_keys:
+            if index_name not in self._global_secondary_indexes_hash_keys:
                 print(message_with_vars(
-                    message="A key_name was not the primary_index key_name, and was not found in the global_secondary_indexes"
-                            "hash_keys. Database query not executed, and None is being returned.",
-                    vars_dict={"primary_index.hash_key_name": self.primary_index.hash_key_name,
-                               "_global_secondary_indexes_hash_keys": self._global_secondary_indexes_hash_keys,
-                               "key_name": key_name, "key_value": key_value, "fields_to_get": fields_to_get}
+                    message="A index_name was not the primary_index index_name, and was not found in the "
+                            "global_secondary_indexes hash_keys. Database query not executed, and None is being returned.",
+                    vars_dict={
+                        'primary_index.hash_key_name': self.primary_index.hash_key_name,
+                        '_global_secondary_indexes_hash_keys': self._global_secondary_indexes_hash_keys,
+                        'index_name': index_name, 'key_value': key_value, "fields_paths_elements": fields_paths_elements
+                    }
                 ))
                 return None
             else:
                 response_items: Optional[List[dict]] = self.query_by_key(
-                    index_name=key_name, key_name=key_name, key_value=key_value,
-                    fields_to_get=fields_to_get, query_limit=1
+                    index_name=index_name, key_value=key_value,
+                    fields_paths_elements=fields_paths_elements, query_limit=1
                 ).items
                 if isinstance(response_items, list) and len(response_items) > 0:
                     return response_items[0]
                 else:
                     return None
 
-    def get_data_in_path_target(self, key_name: str, key_value: str, target_path_elements: List[DatabasePathElement],
-                                num_keys_to_navigation_into: int) -> Optional[any]:
-        target_field = self._database_path_elements_to_dynamodb_target_string(database_path_elements=target_path_elements)
-        response_item = self._get_or_query_single_item(key_name=key_name, key_value=key_value, fields_to_get=[target_field])
-        if response_item is not None:
-            for i, path_element in enumerate(target_path_elements):
-                if i + 1 > num_keys_to_navigation_into or (not isinstance(response_item, dict)):
+    def navigate_into_data_with_field_path_elements(
+            self, data: Any, field_path_elements: List[DatabasePathElement], num_keys_to_navigation_into: int
+    ) -> Optional[Any]:
+        if data is not None:
+            num_field_path_elements = len(field_path_elements)
+            for i, path_element in enumerate(field_path_elements):
+                if i + 1 > num_keys_to_navigation_into:
                     break
+                elif i > 0 and field_path_elements[i - 1].default_type == list:
+                    data = data[0]
+                else:
+                    if not isinstance(data, dict):
+                        break
 
-                response_item = response_item.get(path_element.element_key, None)
-                if response_item is None:
-                    # If the response_item is None, it means that one key has not been found,
-                    # so we need to break the loop in order to try to call get on a None object,
-                    # and then we will return the response_item, so we will return None.
-                    break
-        return response_item
+                    if path_element.default_type == set and data == {} and num_field_path_elements > i + 1:
+                        data = field_path_elements[i + 1].element_key
+                        break
 
-    def get_value_in_path_target(self, key_name: str, key_value: str, target_path_elements: List[DatabasePathElement]) -> Optional[any]:
-        return self.get_data_in_path_target(
-            key_name=key_name, key_value=key_value,
-            target_path_elements=target_path_elements,
-            num_keys_to_navigation_into=len(target_path_elements)
+                    data = data.get(path_element.element_key, None)
+                    if data is None:
+                        # If the response_item is None, it means that one key has not been found,
+                        # so we need to break the loop in order to try to call get on a None object,
+                        # and then we will return the response_item, so we will return None.
+                        break
+        return data
+
+    def get_data_in_path_target(
+            self, index_name: str, key_value: str,
+            field_path_elements: List[DatabasePathElement],
+            num_keys_to_navigation_into: int
+    ) -> Optional[any]:
+
+        response_item = self._get_or_query_single_item(
+            index_name=index_name, key_value=key_value,
+            fields_paths_elements=[field_path_elements]
+        )
+        return self.navigate_into_data_with_field_path_elements(
+            data=response_item, field_path_elements=field_path_elements,
+            num_keys_to_navigation_into=num_keys_to_navigation_into
         )
 
-    def get_item_in_path_target(self, key_name: str, key_value: str, target_path_elements: List[DatabasePathElement]) -> Optional[dict]:
+    def get_value_in_path_target(self, index_name: str, key_value: str, field_path_elements: List[DatabasePathElement]) -> Optional[any]:
         return self.get_data_in_path_target(
-            key_name=key_name, key_value=key_value,
-            target_path_elements=target_path_elements,
-            num_keys_to_navigation_into=len(target_path_elements) - 1
+            index_name=index_name, key_value=key_value,
+            field_path_elements=field_path_elements,
+            num_keys_to_navigation_into=len(field_path_elements)
+        )
+
+    def get_item_in_path_target(self, index_name: str, key_value: str, field_path_elements: List[DatabasePathElement]) -> Optional[dict]:
+        return self.get_data_in_path_target(
+            index_name=index_name, key_value=key_value,
+            field_path_elements=field_path_elements,
+            num_keys_to_navigation_into=len(field_path_elements) - 1
         )
 
     def get_data_from_multiple_fields_in_path_target(
-            self, key_name: str, key_value: str,
-            targets_paths_elements: Dict[str, List[DatabasePathElement]],
-            num_keys_to_stop_at_before_reaching_end_of_item: int
+            self, key_value: str, fields_paths_elements: Dict[str, List[DatabasePathElement]],
+            num_keys_to_stop_at_before_reaching_end_of_item: int, index_name: Optional[str] = None
     ) -> Optional[Any]:
 
-        fields_to_get: List[str] = list()
-        for path_elements_key, path_elements_item in targets_paths_elements.items():
-            fields_to_get.append(self._database_path_elements_to_dynamodb_target_string(database_path_elements=path_elements_item))
-
-        response_item = self._get_or_query_single_item(key_name=key_name, key_value=key_value, fields_to_get=fields_to_get)
+        response_item = self._get_or_query_single_item(
+            index_name=index_name, key_value=key_value,
+            fields_paths_elements=list(fields_paths_elements.values())
+        )
         if response_item is not None:
             output_dict: Dict[str, Any] = dict()
 
-            for path_elements_key, path_elements_item in targets_paths_elements.items():
+            for path_elements_key, path_elements_item in fields_paths_elements.items():
                 if len(path_elements_item) > 0:
                     num_keys_to_navigation_into = len(path_elements_item) - num_keys_to_stop_at_before_reaching_end_of_item
                     first_path_element_item_element_key = path_elements_item[0].element_key
@@ -661,30 +708,20 @@ class DynamoDbCoreAdapter:
             return output_dict
         return None
 
-    def get_values_in_multiple_path_target(self, key_name: str, key_value: str, targets_paths_elements: Dict[str, List[DatabasePathElement]]):
+    def get_values_in_multiple_path_target(self, index_name: str, key_value: str, fields_paths_elements: Dict[str, List[DatabasePathElement]]):
         return self.get_data_from_multiple_fields_in_path_target(
-            key_name=key_name, key_value=key_value,
-            targets_paths_elements=targets_paths_elements,
+            index_name=index_name, key_value=key_value,
+            fields_paths_elements=fields_paths_elements,
             num_keys_to_stop_at_before_reaching_end_of_item=0
         )
 
-    def get_items_in_multiple_path_target(self, key_name: str, key_value: str, targets_paths_elements: Dict[str, List[DatabasePathElement]]):
+    def get_items_in_multiple_path_target(self, index_name: str, key_value: str, fields_paths_elements: Dict[str, List[DatabasePathElement]]):
         return self.get_data_from_multiple_fields_in_path_target(
-            key_name=key_name, key_value=key_value,
-            targets_paths_elements=targets_paths_elements,
+            index_name=index_name, key_value=key_value,
+            fields_paths_elements=fields_paths_elements,
             num_keys_to_stop_at_before_reaching_end_of_item=1
         )
 
-
-    @staticmethod
-    def _database_path_elements_to_dynamodb_target_string(database_path_elements: List[DatabasePathElement]) -> str:
-        target_string = ""
-        for i, path_element in enumerate(database_path_elements):
-            if i > 0:
-                target_string += "."
-            target_string += path_element.element_key
-        return target_string
-
     @staticmethod
     def _add_to_filter_expression(expression, condition):
         if expression is None:
@@ -692,7 +729,7 @@ class DynamoDbCoreAdapter:
         return expression & condition
 
     @staticmethod
-    def _fields_to_get_to_expressions(fields_to_get: List[str]) -> dict:
+    def _fields_paths_elements_to_expressions(fields_paths_elements: List[List[DatabasePathElement]]) -> dict:
         output_kwargs = {}
         expression_attribute_names = {}
         projection_expression = ""
@@ -705,18 +742,25 @@ class DynamoDbCoreAdapter:
         # otherwise it will not work. So, we define an 'id' for each path of each attribute name
         # (f"#f{i_field}_{i_path}") we add our path as the dict value, and then we build the condition
         # expression to use the ids of our attributes names.
-        for i_field, field in enumerate(fields_to_get):
-            field: str  # PyCharm did not recognized the field variable as a string ;)
-            path_elements = field.split(".")
+        for i_field, field_elements in enumerate(fields_paths_elements):
+            for i_path_element, path_element in enumerate(field_elements):
+                if i_path_element > 0:
+                    last_field_path_element = field_elements[i_path_element - 1]
+                    if last_field_path_element.default_type == list:
+                        projection_expression += f'[{path_element.element_key}]'
+                        # We set the index of a list directly inside the projection expression, because a variable can
+                        # be passed the expression_attribute_names only if it is a str, where as our index will be an int.
+                        continue
+                    elif last_field_path_element.default_type == set:
+                        raise Exception("Selection by index in set type's not yet supported")
 
-            for i_path, path in enumerate(path_elements):
-                current_field_path_expression_name = f"#f{i_field}_{i_path}"
-                expression_attribute_names[current_field_path_expression_name] = path
-                projection_expression += current_field_path_expression_name
-                if i_path + 1 < len(path_elements):
-                    projection_expression += "."
+                # If we reach this point, the continue keyword has not been triggered, which means that the current
+                # path_element has not yet been handled by a special case handler, like an list index handler.
+                current_field_path_expression_name = f"#f{i_field}_{i_path_element}"
+                expression_attribute_names[current_field_path_expression_name] = path_element.element_key
+                projection_expression += f'{"." if i_path_element > 0 else ""}{current_field_path_expression_name}'
 
-            if i_field + 1 < len(fields_to_get):
+            if i_field + 1 < len(fields_paths_elements):
                 projection_expression += ", "
 
         if len(expression_attribute_names) > 0:
