@@ -1,11 +1,27 @@
+import re
 from typing import List, Optional, Any, Dict, _GenericAlias, Tuple
-
-from StructNoSQL.dummy_object import DummyObject
 from StructNoSQL.dynamodb.models import DatabasePathElement
+from StructNoSQL.exceptions import InvalidFieldNameException, UsageOfUntypedSetException
 from StructNoSQL.practical_logger import message_with_vars
 from StructNoSQL.query import Query
 
 
+FIELD_NAME_RESTRICTED_CHARS_LIST = ['[', ']', '{', '}']
+FIELD_NAME_RESTRICTED_CHARS_EXPRESSION = r'(\[|\]|\{|\})'
+
+
+def _raise_if_field_name_is_invalid(field_name: str):
+    match: Optional[List[tuple]] = re.findall(pattern=FIELD_NAME_RESTRICTED_CHARS_EXPRESSION, string=field_name)
+    if match is not None and len(match) > 0:
+        raise InvalidFieldNameException(message_with_vars(
+            message="A field name was using one or multiple restricted chars",
+            vars_dict={
+                'fieldName': field_name, 'restrictedCharsMatch': match,
+                'FIELD_NAME_RESTRICTED_CHARS_LIST': FIELD_NAME_RESTRICTED_CHARS_LIST
+            }
+        ))
+
+
 class BaseDataModel:
     def __init__(self):
         self.childrens_map: Optional[dict] = None
@@ -36,8 +52,8 @@ class TableDataModel(MapModel):
 class BaseItem:
     _table = None
     _database_path: Optional[List[DatabasePathElement]] = None
-    _dict_key_expected_type: Optional[type] = None
-    _dict_items_excepted_type: Optional[type or MapModel] = None
+    _key_expected_type: Optional[type] = None
+    _items_excepted_type: Optional[type or MapModel] = None
     # We set the _database_path as static, so that the assign_internal_mapping_from_class can setup the path only once,
     # only by having access to the inheritor class type, not even the instance. Yet, when set the _database_path
     # statically, the value is not attributed to the BaseField class (which would cause to have multiple classes override
@@ -66,11 +82,29 @@ class BaseItem:
                 if alias_variable_name == "Dict":
                     self._field_type = dict
                     self._default_field_type = dict
-                    self._dict_key_expected_type = alias_args[0]
-                    self._dict_items_excepted_type = alias_args[1]
-
+                    self._key_expected_type = alias_args[0]
+                    self._items_excepted_type = alias_args[1]
+                elif alias_variable_name == "Set":
+                    self._field_type = set
+                    self._default_field_type = set
+                    self._items_excepted_type = alias_args[0]
+                    # todo: rename the _items_excepted_type variable
                 elif alias_variable_name == "List":
-                    raise Exception(f"List not yet implemented.")
+                    self._field_type = list
+                    self._default_field_type = list
+                    self._items_excepted_type = alias_args[0]
+                    # todo: rename the _items_excepted_type variable
+
+        elif self._field_type == dict:
+            # Handle an untyped dict
+            self._items_excepted_type = Any
+            self._key_expected_type = Any
+        elif self._field_type == list:
+            # Handle an untyped list
+            self._items_excepted_type = Any
+        elif self._field_type == set:
+            # Raise on an untyped set
+            raise UsageOfUntypedSetException()
 
     def validate_data(self) -> Tuple[Optional[Any], bool]:
         from StructNoSQL.validator import validate_data
@@ -86,7 +120,7 @@ class BaseItem:
         # self.validate_data()
         # print("Finished data validation.")
 
-    def query(self, key_name: str, key_value: str, fields_to_get: List[str], index_name: Optional[str] = None, query_kwargs: Optional[dict] = None) -> Query:
+    def query(self, key_value: str, fields_paths_elements: List[str], index_name: Optional[str] = None, query_kwargs: Optional[dict] = None) -> Query:
         """for path_element in self._database_path:
             if "$key$:" in path_element.element_key:
                 variable_name = path_element.element_key.replace("$key$:", "")
@@ -108,7 +142,7 @@ class BaseItem:
         self._query = Query(
             variable_validator=self, table=self._table,
             target_database_path=self._database_path,
-            key_name=key_name, key_value=key_value, index_name=index_name
+            index_name=index_name, key_value=key_value,
         )
         return self._query
 
@@ -136,12 +170,12 @@ class BaseItem:
         return self._default_field_type
 
     @property
-    def dict_key_expected_type(self) -> Optional[type]:
-        return self._dict_key_expected_type
+    def key_expected_type(self) -> Optional[type]:
+        return self._key_expected_type
 
     @property
-    def dict_items_excepted_type(self) -> Optional[type or MapModel]:
-        return self._dict_items_excepted_type
+    def items_excepted_type(self) -> Optional[type or MapModel]:
+        return self._items_excepted_type
 
     @property
     def database_path(self) -> Optional[List[DatabasePathElement]]:
@@ -151,12 +185,25 @@ class BaseItem:
     def table(self):
         return self._table
 
+    @staticmethod
+    def instantiate_default_value_type(value_type: type) -> Optional[Any]:
+        if value_type == Any:
+            return None
+        else:
+            try:
+                return value_type()
+            except Exception as e:
+                print(e)
+                return None
+
 
 class BaseField(BaseItem):
     def __init__(self, name: str, field_type: Optional[Any] = None, required: Optional[bool] = False, not_modifiable: Optional[bool] = False,
                  custom_default_value: Optional[Any] = None, key_name: Optional[str] = None, max_nested_depth: Optional[int] = 32):
         super().__init__(field_type=field_type if field_type is not None else Any, custom_default_value=custom_default_value)
         self._name = name
+        if self._name is not None:
+            _raise_if_field_name_is_invalid(field_name=self._name)
         self._required = required
         self._key_name = None
 
@@ -169,8 +216,7 @@ class BaseField(BaseItem):
             raise Exception(f"Not modifiable not yet implemented")
 
         if key_name is not None:
-            insta = isinstance(field_type, (tuple, list))
-            if field_type is dict or type(field_type) is _GenericAlias:
+            if field_type in [dict, set, list] or type(field_type) is _GenericAlias:
                 self._key_name = key_name
             elif isinstance(field_type, (tuple, list)):
                 raise Exception(f"Multiple dictionaries are not yet supported.")
@@ -184,16 +230,16 @@ class BaseField(BaseItem):
                     self._key_name = key_name
                 else:
                     raise Exception(message_with_vars(
-                        "key_name cannot be set on a field that is a tuple or list that does not exclusivly contains dict or Dict items",
-                        vars_dict={"fieldName": name, "fieldType": field_type, "keyName": key_name}
+                        message="key_name cannot be set on a field that is a tuple or list that does not exclusively contains dict or Dict items",
+                        vars_dict={'fieldName': name, 'fieldType': field_type, 'keyName': key_name}
                     ))
             else:
                 raise Exception(message_with_vars(
-                    "key_name cannot be set on a field that is not of type dict or Dict",
-                    vars_dict={"fieldName": name, "fieldType": field_type, "keyName": key_name}
+                    message="key_name cannot be set on a field that is not of type dict, Dict, list, List, set or Set",
+                    vars_dict={'fieldName': name, 'fieldType': field_type, 'keyName': key_name}
                 ))
         else:
-            if field_type == dict or type(field_type) == _GenericAlias:
+            if field_type in [dict, set] or type(field_type) == _GenericAlias:
                 self._key_name = f"{name}Key"
 
     def populate(self, value: any):
@@ -202,15 +248,15 @@ class BaseField(BaseItem):
     @property
     def dict_item(self):
         """ :return: BaseField """
-        if self._field_type == dict and self.dict_items_excepted_type is not None:
-            map_item = MapItem(parent_field=self, field_type=self.dict_items_excepted_type, model_type=self.dict_items_excepted_type)
+        if self._field_type == dict and self.items_excepted_type is not None:
+            map_item = MapItem(parent_field=self, field_type=self.items_excepted_type, model_type=self.items_excepted_type)
             return map_item
         else:
             raise Exception(message_with_vars(
                 message="Tried to access dict item of a field that was not of type dict, "
                         "Dict or did not properly received the expected type of the dict items.",
                 vars_dict={"fieldName": self.field_name, "fieldType": self.field_type,
-                           "dictItemsExceptedType": self.dict_items_excepted_type}
+                           "dictItemsExceptedType": self.items_excepted_type}
             ))
 
     def post(self, value: any):
@@ -234,7 +280,7 @@ class BaseField(BaseItem):
         return self._max_nested
 
     def copy(self):
-        # return BaseField(name=self._name, field_type=self._field_type, required=self._required, custom_default_value=self._custom_default_value, key_name=self._key_name, max_nested_depth=self._max_nested)
+        # return BaseField(name=self._name, field_type=self._field_type, required=self._required, custom_default_value=self._custom_default_value, index_name=self._key_name, max_nested_depth=self._max_nested)
         from copy import copy
         return copy(self)
 
@@ -242,12 +288,12 @@ class MapItem(BaseField):
     _default_primitive_type = dict
 
     def __init__(self, parent_field: BaseField, model_type: type, field_type: type):
-        super().__init__(name=None, field_type=field_type, custom_default_value=field_type())
+        super().__init__(name=None, field_type=field_type, custom_default_value=BaseItem.instantiate_default_value_type(field_type))
         self.map_model = model_type
 
         from StructNoSQL.table import make_dict_key_var_name, try_to_get_primitive_default_type_of_item
         element_key = make_dict_key_var_name(parent_field.key_name)
-        default_type = try_to_get_primitive_default_type_of_item(parent_field.dict_items_excepted_type)
+        default_type = try_to_get_primitive_default_type_of_item(parent_field.items_excepted_type)
         database_path_element = DatabasePathElement(element_key=element_key, default_type=default_type)
         self._database_path = [*parent_field.database_path, database_path_element]
         self._table = parent_field.table
