@@ -1,4 +1,4 @@
-from typing import Optional, List, Dict, Any, Set
+from typing import Optional, List, Dict, Any, Set, Tuple
 from copy import copy
 
 from StructNoSQL.dynamodb.dynamodb_core import DynamoDbCoreAdapter, PrimaryIndex, GlobalSecondaryIndex, DynamoDBMapObjectSetter, Response
@@ -45,6 +45,7 @@ class BaseTable:
             global_secondary_indexes=global_secondary_indexes,
             create_table=auto_create_table
         )
+        self._primary_index_name = primary_index.index_custom_name or primary_index.hash_key_name
 
         if not isinstance(data_model, type):
             self._model = data_model
@@ -55,6 +56,10 @@ class BaseTable:
         self.processed_class_types: Set[type] = set()
         assign_internal_mapping_from_class(table=self, class_instance=self._model)
 
+    @property
+    def primary_index_name(self) -> str:
+        return self._primary_index_name
+
     @property
     def model(self) -> TableDataModel:
         return self._model
@@ -93,56 +98,65 @@ class BaseTable:
         else:
             return False
 
-    def get_single_field_item_from_single_item(self, key_name: str, key_value: str, field_to_get: str, query_kwargs: Optional[dict] = None) -> Any:
-        response_data = self.dynamodb_client.get_item_in_path_target(
-            key_name=key_name, key_value=key_value,
-            target_path_elements=process_and_make_single_rendered_database_path(
-                field_path=field_to_get, fields_switch=self.fields_switch, query_kwargs=query_kwargs
-            )
+    def get_field(self, key_value: str, field_path: str, query_kwargs: Optional[dict] = None, index_name: Optional[str] = None) -> Any:
+        field_path_elements, has_multiple_fields_path = process_and_make_single_rendered_database_path(
+            field_path=field_path, fields_switch=self.fields_switch, query_kwargs=query_kwargs
         )
-        return response_data
-
-    def get_single_field_value_from_single_item(self, key_name: str, key_value: str, field_to_get: str, query_kwargs: Optional[dict] = None) -> Any:
-        response_data = self.dynamodb_client.get_value_in_path_target(
-            key_name=key_name, key_value=key_value,
-            target_path_elements=process_and_make_single_rendered_database_path(
-                field_path=field_to_get, fields_switch=self.fields_switch, query_kwargs=query_kwargs
+        if has_multiple_fields_path is not True:
+            field_path_elements: List[DatabasePathElement]
+            response_data = self.dynamodb_client.get_value_in_path_target(
+                index_name=index_name or self.primary_index_name,
+                key_value=key_value, field_path_elements=field_path_elements
             )
-        )
-        return response_data
+            return response_data
+        else:
+            field_path_elements: Dict[str, List[DatabasePathElement]]
+            response_data = self.dynamodb_client.get_values_in_multiple_path_target(
+                index_name=index_name or self.primary_index_name,
+                key_value=key_value, fields_paths_elements=field_path_elements
+            )
+            return response_data
 
-    def _getters_to_database_paths(self, getters: Dict[str, FieldGetter]) -> Dict[str, List[DatabasePathElement]]:
+    def _getters_to_database_paths(self, getters: Dict[str, FieldGetter]) -> Dict[str, List[DatabasePathElement] or Dict[str, List[DatabasePathElement]]]:
         getters_database_paths: Dict[str, List[DatabasePathElement]] = dict()
         for getter_key, getter_item in getters.items():
-            getters_database_paths[getter_key] = process_and_make_single_rendered_database_path(
-                field_path=getter_item.target_path, fields_switch=self.fields_switch, query_kwargs=getter_item.query_kwargs
+            getter_field_path_elements, getter_has_multiple_fields_path = process_and_make_single_rendered_database_path(
+                field_path=getter_item.field_path, fields_switch=self.fields_switch, query_kwargs=getter_item.query_kwargs
             )
+            getter_field_path_elements: List[DatabasePathElement]
+            getters_database_paths[getter_key] = getter_field_path_elements
+            # No matter if getter_has_multiple_fields_path is True, we still add the getters_database_paths the same way. We will perform
+            # different logic later on, depending on if we have a Dict[str, List[DatabasePathElement]] or simple a List[DatabasePathElement]
         return getters_database_paths
 
-    def get_multiple_fields_items_from_single_item(self, key_name: str, key_value: str, getters: Dict[str, FieldGetter]) -> Optional[dict]:
-        getters_database_paths = self._getters_to_database_paths(getters=getters)
-        response_data = self.dynamodb_client.get_items_in_multiple_path_target(
-            key_name=key_name, key_value=key_value, targets_paths_elements=getters_database_paths
-        )
-        return response_data
-
-    def get_multiple_fields_values_from_single_item(self, key_name: str, key_value: str, getters: Dict[str, FieldGetter]) -> Optional[dict]:
+    def get_multiple_fields(self, key_value: str, getters: Dict[str, FieldGetter], index_name: Optional[str] = None) -> Optional[dict]:
         getters_database_paths = self._getters_to_database_paths(getters=getters)
         response_data = self.dynamodb_client.get_values_in_multiple_path_target(
-            key_name=key_name, key_value=key_value, targets_paths_elements=getters_database_paths
+            index_name=index_name or self.primary_index_name,
+            key_value=key_value, fields_paths_elements=getters_database_paths,
         )
         return response_data
 
-    def query(self, key_name: str, key_value: str, fields_to_get: List[str], index_name: Optional[str] = None, limit: Optional[int] = None,
-              query_kwargs: Optional[dict] = None, filter_expression: Optional[Any] = None, **additional_kwargs) -> Optional[List[Any]]:
-        # rendered_fields_paths = make_rendered_fields_paths(fields_paths=fields_to_get, query_kwargs=query_kwargs)
+    # todo: deprecated
+    """
+    def query(self, key_value: str, fields_paths: List[str], query_kwargs: Optional[dict] = None, limit: Optional[int] = None,
+              filter_expression: Optional[Any] = None,  index_name: Optional[str] = None, **additional_kwargs) -> Optional[List[Any]]:
         fields_paths_objects = process_and_get_fields_paths_objects_from_fields_paths(
-            fields_paths=fields_to_get, fields_switch=self.fields_switch
+            fields_paths=fields_paths, fields_switch=self.fields_switch
         )
+        query_field_path_elements: List[List[DatabasePathElement]] = list()
+        for field_path in fields_paths:
+            field_path_elements, has_multiple_fields_path = process_and_make_single_rendered_database_path(
+                field_path=field_path, fields_switch=self.fields_switch, query_kwargs=query_kwargs
+            )
+            query_field_path_elements.append(field_path_elements)
+
         response = self.dynamodb_client.query_by_key(
-            key_name=key_name, key_value=key_value,
-            fields_to_get=fields_to_get, index_name=index_name, query_limit=limit,
-            filter_expression=filter_expression, **additional_kwargs
+            index_name=index_name or self.primary_index_name,
+            index_name=key_name, key_value=key_value,
+            fields_paths_elements=query_field_path_elements,
+            query_limit=limit, filter_expression=filter_expression, 
+            **additional_kwargs
         )
         if response is not None:
             for current_item in response.items:
@@ -153,86 +167,161 @@ class BaseTable:
                             if matching_field_path_object.database_path is not None:
                                 matching_field_path_object.populate(value=current_item_value)
                                 current_item[current_item_key], valid = matching_field_path_object.validate_data()
+                                # todo: remove this non centralized response validation system
             return response.items
         else:
             return None
+    """
 
-    def set_update_one_field(self, key_name: str, key_value: str, target_field: str, value_to_set: Any,
-                             index_name: Optional[str] = None, query_kwargs: Optional[dict] = None) -> bool:
-        validated_data, valid, target_path_elements = process_validate_data_and_make_single_rendered_database_path(
-            field_path=target_field, fields_switch=self.fields_switch, query_kwargs=query_kwargs, data_to_validate=value_to_set
+    def update_field(self, key_value: str, field_path: str, value_to_set: Any,
+                     query_kwargs: Optional[dict] = None, index_name: Optional[str] = None) -> bool:
+        validated_data, valid, field_path_elements = process_validate_data_and_make_single_rendered_database_path(
+            field_path=field_path, fields_switch=self.fields_switch, query_kwargs=query_kwargs, data_to_validate=value_to_set
         )
-        if valid is True and target_path_elements is not None:
+        if valid is True and field_path_elements is not None:
             response = self.dynamodb_client.set_update_data_element_to_map(
-                key_name=key_name, key_value=key_value, value=validated_data,
-                target_path_elements=target_path_elements
+                index_name=index_name or self.primary_index_name,
+                key_value=key_value, value=validated_data,
+                field_path_elements=field_path_elements
             )
             return True if response is not None else False
         return False
 
-    def set_update_multiple_fields(self, key_name: str, key_value: str, setters: List[FieldSetter or UnsafeFieldSetter]) -> bool:
+    def update_multiple_fields(self, key_value: str, setters: List[FieldSetter or UnsafeFieldSetter], index_name: Optional[str] = None) -> bool:
         dynamodb_setters: List[DynamoDBMapObjectSetter] = list()
         for current_setter in setters:
             if isinstance(current_setter, FieldSetter):
-                validated_data, valid, target_path_elements = process_validate_data_and_make_single_rendered_database_path(
-                    field_path=current_setter.target_path, fields_switch=self.fields_switch,
+                validated_data, valid, field_path_elements = process_validate_data_and_make_single_rendered_database_path(
+                    field_path=current_setter.field_path, fields_switch=self.fields_switch,
                     query_kwargs=current_setter.query_kwargs, data_to_validate=current_setter.value_to_set
                 )
                 if valid is True:
                     dynamodb_setters.append(DynamoDBMapObjectSetter(
-                        target_path_elements=target_path_elements, value_to_set=validated_data
+                        field_path_elements=field_path_elements, value_to_set=validated_data
                     ))
             elif isinstance(current_setter, UnsafeFieldSetter):
-                safe_target_field = process_and_get_field_path_object_from_field_path(
-                    field_path_key=current_setter.safe_base_target_path, fields_switch=self.fields_switch
+                safe_field_path_object, has_multiple_fields_path = process_and_get_field_path_object_from_field_path(
+                    field_path_key=current_setter.safe_base_field_path, fields_switch=self.fields_switch
                 )
+                # todo: add support for multiple fields path
                 if current_setter.unsafe_path_continuation is None:
-                    target_path_elements = safe_target_field.database_path
+                    field_path_elements = safe_field_path_object.database_path
                 else:
-                    target_path_elements = safe_target_field.database_path + current_setter.unsafe_path_continuation
+                    field_path_elements = safe_field_path_object.database_path + current_setter.unsafe_path_continuation
 
                 processed_value_to_set: Any = float_to_decimal_serializer(current_setter.value_to_set)
                 # Since the data is not validated, we pass it to the float_to_decimal_serializer
                 # function (which normally should be called by the data validation function)
 
-                rendered_target_path_elements = make_rendered_database_path(
-                    database_path_elements=target_path_elements,
+                rendered_field_path_elements = make_rendered_database_path(
+                    database_path_elements=field_path_elements,
                     query_kwargs=current_setter.query_kwargs
                 )
                 dynamodb_setters.append(DynamoDBMapObjectSetter(
-                    target_path_elements=rendered_target_path_elements,
+                    field_path_elements=rendered_field_path_elements,
                     value_to_set=processed_value_to_set
                 ))
 
         response = self.dynamodb_client.set_update_multiple_data_elements_to_map(
-            key_name=key_name, key_value=key_value, setters=dynamodb_setters
+            index_name=index_name or self.primary_index_name,
+            key_value=key_value, setters=dynamodb_setters
         )
         return True if response is not None else False
 
-    def remove_single_item_at_path_target(self, key_name: str, key_value: str, target_field: str,
-                                          query_kwargs: Optional[dict] = None) -> bool:
-        response: Optional[Response] = self.dynamodb_client.remove_data_elements_from_map(
-            key_name=key_name, key_value=key_value,
-            targets_path_elements=[process_and_make_single_rendered_database_path(
-                field_path=target_field, fields_switch=self.fields_switch, query_kwargs=query_kwargs
-            )]
-        )
-        return True if response is not None else False
+    def _base_removal(
+            self, retrieve_removed_elements: bool, key_value: str, field_path: str,
+            query_kwargs: Optional[dict] = None, index_name: Optional[str] = None
+    ) -> Tuple[Optional[Response], List[List[DatabasePathElement]]]:
 
-    def remove_multiple_items_at_path_targets(self, key_name: str, key_value: str, removers: List[FieldRemover]) -> bool:
-        if len(removers) > 0:
-            removers_database_paths: List[List[DatabasePathElement]] = list()
-            for current_remover in removers:
-                removers_database_paths.append(
-                    process_and_make_single_rendered_database_path(
-                        field_path=current_remover.target_path,
-                        fields_switch=self.fields_switch,
-                        query_kwargs=current_remover.query_kwargs
-                    )
+        field_path_elements, has_multiple_fields_path = process_and_make_single_rendered_database_path(
+            field_path=field_path, fields_switch=self.fields_switch, query_kwargs=query_kwargs
+        )
+        target_path_elements = [field_path_elements] if has_multiple_fields_path is not True else list(field_path_elements.values())
+        # The remove_data_elements_from_map function expect a List[List[DatabasePathElement]]. If we have a single field_path, we wrap the field_path_elements
+        # inside a list. And if we have multiple fields_paths (which will be structured inside a dict), we turn the convert the values of the dict to a list.
+
+        return self.dynamodb_client.remove_data_elements_from_map(
+            index_name=index_name or self.primary_index_name,
+            key_value=key_value, targets_path_elements=target_path_elements,
+            retrieve_removed_elements=retrieve_removed_elements
+        ), target_path_elements
+
+    def remove_field(self, key_value: str, field_path: str, query_kwargs: Optional[dict] = None, index_name: Optional[str] = None) -> Optional[Any]:
+        response, all_fields_items_path_elements = self._base_removal(
+            retrieve_removed_elements=True, key_value=key_value,
+            field_path=field_path, query_kwargs=query_kwargs, index_name=index_name
+        )
+        if response is not None and response.attributes is not None:
+            if not len(all_fields_items_path_elements) > 0:
+                return None
+            elif len(all_fields_items_path_elements) == 1:
+                field_path_elements = all_fields_items_path_elements[0]
+                removed_item_data = self.dynamodb_client.navigate_into_data_with_field_path_elements(
+                    data=response.attributes, field_path_elements=field_path_elements,
+                    num_keys_to_navigation_into=len(field_path_elements)
                 )
+                return removed_item_data
+            else:
+                removed_items_values: Dict[str, Any] = dict()
+                for field_path_elements in all_fields_items_path_elements:
+                    # Even the remove_field function can potentially remove multiple
+                    # field_path_elements if the field_path expression is selecting multiple fields.
+                    last_path_element = field_path_elements[len(field_path_elements) - 1]
+                    removed_items_values[last_path_element.element_key] = self.dynamodb_client.navigate_into_data_with_field_path_elements(
+                        data=response.attributes, field_path_elements=field_path_elements,
+                        num_keys_to_navigation_into=len(field_path_elements)
+                    )
+                return removed_items_values
+        return None
 
-            response: Optional[Response] = self.dynamodb_client.remove_data_elements_from_map(
-                key_name=key_name, key_value=key_value, targets_path_elements=removers_database_paths
+    def delete_field(self, key_value: str, field_path: str, query_kwargs: Optional[dict] = None, index_name: Optional[str] = None) -> bool:
+        response, _ = self._base_removal(
+            retrieve_removed_elements=False, key_value=key_value,
+            field_path=field_path, query_kwargs=query_kwargs, index_name=index_name
+        )
+        return True if response is not None else False
+
+    def _base_multi_removal(
+            self, retrieve_removed_elements: bool, key_value: str,
+            removers: List[FieldRemover], index_name: Optional[str] = None
+    ) -> Optional[Response]:
+
+        removers_database_paths: List[List[DatabasePathElement]] = list()
+        for current_remover in removers:
+            field_path_elements, has_multiple_fields_path = process_and_make_single_rendered_database_path(
+                field_path=current_remover.field_path, fields_switch=self.fields_switch, query_kwargs=current_remover.query_kwargs
+            )
+            if has_multiple_fields_path is not True:
+                field_path_elements: List[DatabasePathElement]
+                removers_database_paths.append(field_path_elements)
+            else:
+                field_path_elements: Dict[str, List[DatabasePathElement]]
+                for field_paths_elements_item in field_path_elements.values():
+                    removers_database_paths.append(field_paths_elements_item)
+
+        return self.dynamodb_client.remove_data_elements_from_map(
+            index_name=index_name or self.primary_index_name,
+            key_value=key_value, targets_path_elements=removers_database_paths,
+            retrieve_removed_elements=retrieve_removed_elements
+        )
+
+    def remove_multiple_fields(self, key_value: str, removers: List[FieldRemover], index_name: Optional[str] = None) -> Optional[Any]:
+        if len(removers) > 0:
+            response: Optional[Response] = self._base_multi_removal(
+                retrieve_removed_elements=True, key_value=key_value,
+                removers=removers, index_name=index_name
+            )
+            return response.items if response is not None and response.items is not None else None
+        else:
+            # If no remover has been specified, we do not run the database
+            # operation, and since no value has been removed, we return None.
+            return None
+
+    def delete_multiple_fields(self, key_value: str, removers: List[FieldRemover], index_name: Optional[str] = None) -> bool:
+        if len(removers) > 0:
+            response: Optional[Response] = self._base_multi_removal(
+                retrieve_removed_elements=False, key_value=key_value,
+                removers=removers, index_name=index_name
             )
             return True if response is not None else False
         else:
@@ -321,7 +410,7 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
                     required_fields.append(variable_item)
 
                 current_field_path += f"{variable_item.field_name}" if len(current_field_path) == 0 else f".{variable_item.field_name}"
-                field_is_valid = table.fields_switch.set(key=current_field_path, item=variable_item)
+                field_is_valid = table.fields_switch.set(key=current_field_path, item=copy(variable_item))
                 if field_is_valid is True:
                     output_mapping[variable_item.field_name] = assign_internal_mapping_from_class(
                         table=table, class_type=variable_item.map_model, nested_field_path=current_field_path,
@@ -345,13 +434,13 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
                 current_field_path += f"{variable_item.field_name}" if len(current_field_path) == 0 else f".{variable_item.field_name}"
                 field_is_valid = table.fields_switch.set(key=current_field_path, item=copy(variable_item))
                 if field_is_valid is True:
-                    if variable_item.dict_items_excepted_type is not None:
+                    if variable_item.items_excepted_type is not None:
                         from StructNoSQL import ActiveSelf
-                        if variable_item.dict_items_excepted_type is ActiveSelf:
-                            variable_item._dict_items_excepted_type = class_type
+                        if variable_item.items_excepted_type is ActiveSelf:
+                            variable_item._items_excepted_type = class_type
 
-                        item_default_type = try_to_get_primitive_default_type_of_item(item_type=variable_item.dict_items_excepted_type)
-                        item_key_name = make_dict_key_var_name(key_name=variable_item.key_name)
+                        item_default_type = try_to_get_primitive_default_type_of_item(item_type=variable_item.items_excepted_type)
+                        item_key_name = make_dict_key_var_name(variable_item.key_name)
 
                         if "{i}" in variable_item.key_name:
                             if is_nested is not True:
@@ -365,7 +454,7 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
                                     map_item = MapItem(
                                         parent_field=nested_variable_item,
                                         field_type=nested_variable_item.default_field_type,
-                                        model_type=nested_variable_item.dict_items_excepted_type
+                                        model_type=nested_variable_item.items_excepted_type
                                     )
                                     current_nested_field_path += f".{variable_item.field_name}"
                                     current_nested_field_path += ".{{" + item_rendered_key_name + "}}"
@@ -377,9 +466,9 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
                                     ))
                                     field_is_valid = table.fields_switch.set(key=current_nested_field_path, item=map_item)
                                     if field_is_valid is True:
-                                        if variable_item.dict_items_excepted_type not in PRIMITIVE_TYPES:
+                                        if variable_item.items_excepted_type not in PRIMITIVE_TYPES:
                                             output_mapping[item_key_name] = assign_internal_mapping_from_class(
-                                                table=table, class_type=variable_item.dict_items_excepted_type,
+                                                table=table, class_type=variable_item.items_excepted_type,
                                                 nested_field_path=current_nested_field_path,
                                                 current_path_elements=[*current_nested_database_path], is_nested=True
                                             )
@@ -392,14 +481,14 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
                             current_field_path += ".{{" + variable_item.key_name + "}}"
                             map_item = MapItem(
                                 parent_field=variable_item, field_type=item_default_type,
-                                model_type=variable_item.dict_items_excepted_type
+                                model_type=variable_item.items_excepted_type
                             )
                             field_is_valid = table.fields_switch.set(current_field_path, map_item)
                             if field_is_valid is True:
-                                if variable_item.dict_items_excepted_type not in PRIMITIVE_TYPES:
+                                if variable_item.items_excepted_type not in PRIMITIVE_TYPES:
                                     new_database_dict_item_path_element = DatabasePathElement(element_key=item_key_name, default_type=item_default_type)
                                     output_mapping[item_key_name] = assign_internal_mapping_from_class(
-                                        table=table, class_type=variable_item.dict_items_excepted_type, nested_field_path=current_field_path,
+                                        table=table, class_type=variable_item.items_excepted_type, nested_field_path=current_field_path,
                                         current_path_elements=[*variable_item.database_path, new_database_dict_item_path_element]
                                     )
 
